<!DOCTYPE html>
<html>
  <head>
    <meta charset="utf-8">
    <meta name="viewport" content="width=device-width">
    <title><%= @title %></title>
    <link rel="stylesheet" href="https://cdn.jsdelivr.net/npm/qunit@<%= qunit_version %>/qunit/qunit.css">
  </head>
  <body>
    <%= yield %>
  </body>
</html>
