# frozen_string_literal: true

require 'active_model'
require 'client_side_validations/active_model'
require 'action_view/models/category'
require 'action_view/models/comment'
require 'action_view/models/format_thing'
require 'action_view/models/post'
