# frozen_string_literal: true

require 'base_helper'
require 'active_record'
require 'client_side_validations/active_record'

require 'active_record/models/user'
require 'active_record/models/guid'
require 'active_record/models/thing'
