# frozen_string_literal: true

# Sanity check to make sure the entire library loads OK

require 'base_helper'
require 'client_side_validations'
require 'client_side_validations/files'
require 'client_side_validations/version'
