<% @title = "client_side_validations test" %>

<div id="qunit"></div>
<div id="qunit-fixture"></div>

<%= script_tag '/vendor/assets/javascripts/rails.validations.js' %>
<%= script_tag "https://cdn.jsdelivr.net/npm/qunit@#{qunit_version}/qunit/qunit.js" %>
<%= script_tag 'settings' %>
<%= script_tag 'validateElement' %>
<%= test :validators, :form_builders, :callbacks %>
